{ File    : $URL: file:///D:/SVN/svnDelphi/trunk/DBIOS/src/DBIOS.dpr $
  Author  : $Author: Deniz Oezmen $
  Changed : $Date: 2020-03-13 10:49:20 +0100 (Fr, 13 Mrz 2020) $
  Revision: $Rev: 542 $

  Tool to (un)scramble Diamond VGA BIOS files; specifically for SpeedSTAR24
  BIOSes.
}
program DBIOS;

{$APPTYPE CONSOLE}

{$R '..\res\version.res' '..\res\version.rc'}

uses
  CmdLine  in 'utils\CmdLine.pas',
  ConOut   in 'utils\ConOut.pas',
  ModInfo  in 'utils\ModInfo.pas',
  SysUtils,
  Windows,
  Classes;

const
  AddressLength = 15;
  DataLength    = 8;
  BIOSSize      = 1 shl AddressLength;

type
  TShuffleMap = array[0..AddressLength - 1] of Byte;
  TMap        = array[0..1 shl AddressLength - 1] of Word;

const
  AddressShuffleMapLow : TShuffleMap = ( 3,  2, 12,  1, 11,  4, 10,  5,
                                         8, 13,  0, 14,  9,  6,  7);
  AddressShuffleMapHigh: TShuffleMap = ( 9,  5, 10,  4, 11,  1, 12,  2,
                                        13,  8,  7,  6,  3, 14,  0);
  DataShuffleMapLow    : TShuffleMap = ( 0,  1,  2,  7,  6,  5,  3,  4,
                                         0,  0,  0,  0,  0,  0,  0);
  DataShuffleMapHigh   : TShuffleMap = ( 7,  6,  5,  4,  3,  2,  1,  0,
                                         0,  0,  0,  0,  0,  0,  0);

var
  AddressMapLow,
  AddressMapLowReverse,
  AddressMapHigh,
  AddressMapHighReverse,
  DataMapLow,
  DataMapLowReverse,
  DataMapHigh,
  DataMapHighReverse   : TMap;

procedure GenerateMap(const ShuffleMap: TShuffleMap; const Len: Byte;
  out Map: TMap; out ReverseMap: TMap);
var
  Bit       : Byte;
  Val,
  ValShifted,
  ValMapped : Word;
begin
  for Val := 0 to 1 shl Len - 1 do
  begin
    ValMapped := 0;

    for Bit := 0 to Len - 1 do
    begin
      if ShuffleMap[Bit] <= Bit then
        ValShifted := (Val and (1 shl Bit)) shr (Bit - ShuffleMap[Bit])
      else
        ValShifted := (Val and (1 shl Bit)) shl (ShuffleMap[Bit] - Bit);

      ValMapped := ValMapped or ValShifted
    end;

    Map[Val] := ValMapped;
    ReverseMap[ValMapped] := Val
  end
end;

procedure UnscrambleBIOS(const InFileName, OutFileName: AnsiString);
var
  InFile,
  OutFile: file;
  i,
  Size   : Longint;
  InBuf,
  OutBuf : array of Byte;
begin
  AssignFile(InFile, InFileName);
  {$I-}
  Reset(InFile, 1);
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn(Format('could not open %s', [InFileName]));
    Exit
  end;

  Size := FileSize(InFile);
  if Size <> BIOSSize then
  begin
    WriteLn(Format('%s is %d bytes large, expected %d bytes',
      [InFileName, Size, BIOSSize]));
    CloseFile(InFile);

    Exit
  end;

  AssignFile(OutFile, OutFileName);
  {$I-}
  Rewrite(OutFile, 1);
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn(Format('could not create %s', [OutFileName]));
    CloseFile(InFile);

    Exit
  end;

  SetLength(InBuf, Size);
  SetLength(OutBuf, Size);
  FillChar(OutBuf[0], Length(OutBuf), 0);

  {$I-}
  BlockRead(InFile, InBuf[0], Length(InBuf));
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn(Format('could not read from %s', [InFileName]));
    CloseFile(InFile);
    CloseFile(OutFile);
    SetLength(InBuf, 0);
    SetLength(OutBuf, 0);

    Exit
  end;

  for i := 0 to High(InBuf) do
    OutBuf[AddressMapLow[i]] := DataMapLow[InBuf[i]];

  {$I-}
  BlockWrite(OutFile, OutBuf[0], Length(OutBuf));
  {$I+}
  if IOResult <> 0 then
    WriteLn(Format('could not write to %s', [OutFileName]));

  CloseFile(InFile);
  CloseFile(OutFile);
  SetLength(InBuf, 0);
  SetLength(OutBuf, 0)
end;

procedure ScrambleBIOS(const InFileName, OutLowFileName,
  OutHighFileName: AnsiString);
var
  InFile,
  OutLowFile,
  OutHighFile: file;
  i,
  Size       : Longint;
  InBuf,
  OutLowBuf,
  OutHighBuf : array of Byte;
begin
  AssignFile(InFile, InFileName);
  {$I-}
  Reset(InFile, 1);
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn(Format('could not open %s', [InFileName]));
    Exit
  end;

  Size := FileSize(InFile);
  if Size <> BIOSSize then
  begin
    WriteLn(Format('%s is %d bytes large, expected %d bytes',
      [InFileName, Size, BIOSSize]));
    CloseFile(InFile);

    Exit
  end;

  AssignFile(OutLowFile, OutLowFileName);
  {$I-}
  Rewrite(OutLowFile, 1);
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn(Format('could not create %s', [OutLowFileName]));
    CloseFile(InFile);

    Exit
  end;

  AssignFile(OutHighFile, OutHighFileName);
  {$I-}
  Rewrite(OutHighFile, 1);
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn(Format('could not create %s', [OutHighFileName]));
    CloseFile(InFile);
    CloseFile(OutLowFile);

    Exit
  end;

  SetLength(InBuf, Size);
  SetLength(OutLowBuf, Size);
  SetLength(OutHighBuf, Size);
  FillChar(OutLowBuf[0], Length(OutLowBuf), 0);
  FillChar(OutHighBuf[0], Length(OutHighBuf), 0);

  {$I-}
  BlockRead(InFile, InBuf[0], Length(InBuf));
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn(Format('could not read from %s', [InFileName]));
    CloseFile(InFile);
    CloseFile(OutLowFile);
    CloseFile(OutHighFile);
    SetLength(InBuf, 0);
    SetLength(OutLowBuf, 0);
    SetLength(OutHighBuf, 0);

    Exit
  end;

  for i := 0 to High(InBuf) do
  begin
    OutLowBuf[AddressMapLowReverse[i]] := DataMapLowReverse[InBuf[i]];
    OutHighBuf[AddressMapHighReverse[i]] :=
      DataMapHighReverse[InBuf[i + (1 - i and 1)]]
  end;

  {$I-}
  BlockWrite(OutLowFile, OutLowBuf[0], Length(OutLowBuf));
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn(Format('could not write to %s', [OutLowFileName]));
    CloseFile(InFile);
    CloseFile(OutLowFile);
    CloseFile(OutHighFile);
    SetLength(InBuf, 0);
    SetLength(OutLowBuf, 0);
    SetLength(OutHighBuf, 0);

    Exit
  end;

  {$I-}
  BlockWrite(OutHighFile, OutHighBuf[0], Length(OutHighBuf));
  {$I+}
  if IOResult <> 0 then
    WriteLn(Format('could not write to %s', [OutHighFileName]));

  CloseFile(InFile);
  CloseFile(OutLowFile);
  CloseFile(OutHighFile);
  SetLength(InBuf, 0);
  SetLength(OutLowBuf, 0);
  SetLength(OutHighBuf, 0)
end;

procedure HandleBIOS;
begin
  GenerateMap(AddressShuffleMapLow, AddressLength, AddressMapLow,
    AddressMapLowReverse);
  GenerateMap(AddressShuffleMapHigh, AddressLength, AddressMapHigh,
    AddressMapHighReverse);
  GenerateMap(DataShuffleMapLow, DataLength, DataMapLow, DataMapLowReverse);
  GenerateMap(DataShuffleMapHigh, DataLength, DataMapHigh, DataMapHighReverse);

  if ParamRec.Unscramble then
    UnscrambleBIOS(ParamRec.NameSpecIn, ParamRec.NameSpecOut1)
  else if ParamRec.Scramble then
    ScrambleBIOS(ParamRec.NameSpecIn, ParamRec.NameSpecOut1,
      ParamRec.NameSpecOut2)
end;

begin
  try
    Command := ProcessParams;

    if (Command and Command_Info) <> 0 then
      WriteModuleInfo;
    if (Command and Command_Invalid) <> 0 then
      WriteLn(Format('Usage: %s [/u <ScrambledLowIn> <UnscrambledOut> | /s <U' +
        'nscrambledIn> <ScrambledLowOut> <ScrambledHighOut>] [/info]',
        [GetModuleNamePrefix]));
    if (Command and Command_Operate) <> 0 then
      HandleBIOS
  except
    on e: Exception do
      WriteLn('Exception: ', e.Message)
  end
end.
