{ File    : $URL: file:///D:/SVN/svnDelphi/trunk/DBIOS/src/utils/CmdLine.pas $
  Author  : $Author: Deniz Oezmen $
  Changed : $Date: 2020-03-13 10:46:05 +0100 (Fr, 13 Mrz 2020) $
  Revision: $Rev: 539 $

  Diamond BIOS tool command line handling.
}
unit CmdLine;

interface

const
  Command_None    = $00000000;
  Command_Invalid = $00000001;
  Command_Info    = $00000002;
  Command_Operate = $00000004;

type
  TParamRec = record
    NameSpecIn  : AnsiString;
    NameSpecOut1: AnsiString;
    NameSpecOut2: AnsiString;
    Unscramble  : Boolean;
    Scramble    : Boolean;
    Info        : Boolean;
  end;

var
  ParamRec: TParamRec;
  Command : Longword;

function ProcessParams: Longword;
procedure WriteModuleInfo;

implementation

uses
  ConOut,
  ModInfo,
  SysUtils;

function ParseTime(const Param: AnsiString; var SetFlag: Boolean;
  var SetLow: Longword; var SetHigh: Longword): Boolean;
var
  TimesStr,
  LowStr,
  HighStr : AnsiString;
  SplitPos,
  LowInt,
  HighInt : Longint;
begin
  Result := False;

  if SetFlag then
    Exit;

  TimesStr := Copy(Param, 7, Length(Param) - 6);

  SplitPos := Pos(',', TimesStr);

  if (SplitPos <= 1) or (SplitPos >= Length(TimesStr)) then
    Exit;

  LowStr := Copy(TimesStr, 1, SplitPos - 1);
  HighStr := Copy(TimesStr, SplitPos + 1, Length(TimesStr) - SplitPos);

  if TryStrToInt(LowStr, LowInt) and TryStrToInt(HighStr, HighInt) then
  begin
    SetFlag := True;
    SetLow := Longword(LowInt);
    SetHigh := Longword(HighInt)
  end
  else
    Exit;

  Result := True
end;

function ProcessParams: Longword;
var
  i         : Longint;
  Param,
  ParamLower: AnsiString;
begin
  Result := Command_None;

  with ParamRec do
  begin
    NameSpecIn   := '';
    NameSpecOut1 := '';
    NameSpecOut2 := '';
    Unscramble   := False;
    Scramble     := False;
    Info         := False;

    // collect information from all parameters
    for i := 1 to ParamCount do
    begin
      Param := ParamStr(i);
      ParamLower := AnsiLowerCase(Param);
      if ParamLower = '/info' then
        Info := True
      else if ParamLower = '/u' then
        Unscramble := True
      else if ParamLower = '/s' then
        Scramble := True
      else if Copy(ParamLower, 1, 1) = '/' then
        // any other parameter looking like a switch is unknown
        Result := Result or Command_Invalid
      else if NameSpecIn = '' then
        NameSpecIn := Param
      else if NameSpecOut1 = '' then
        NameSpecOut1 := Param
      else if NameSpecOut2 = '' then
        NameSpecOut2 := Param
      else
        Result := Result or Command_Invalid
    end;

    // check for info flag
    if Info then
      Result := Result or Command_Info;

    // check for invalid parameter combinations
    if Unscramble and Scramble then
      Result := Result or Command_Invalid;
    if Unscramble and ((NameSpecIn = '') or (NameSpecOut1 = '') or
      (NameSpecOut2 <> '')) then
      Result := Result or Command_Invalid;
    if Scramble and ((NameSpecIn = '') or (NameSpecOut1 = '') or
      (NameSpecOut2 = '')) then
      Result := Result or Command_Invalid;

    // if no invalid parameters have been found, check for minimum operation
    // requirements
    if Result and Command_Invalid = 0 then
    begin
      if Scramble or Unscramble then
        Result := Result or Command_Operate
    end;

    // final check: for the command line to be valid, there has to be at least
    // one task to do (be it actual operation or just printing some info)
    if ((Result and (Command_Info or Command_Operate)) = 0) then
      Result := Result or Command_Invalid
  end
end;

procedure WriteModuleInfo;
var
  i         : Longint;
  ModuleInfo: TModuleInfo;
begin
  ModuleInfo := GetModuleInfo;

  WriteLnColored(TextAttr_White, '#');

  for i := Low(ModuleInfo) to High(ModuleInfo) do
    WriteLnColored(TextAttr_White, Format('| %s', [ModuleInfo[i]]));

  WriteLn
end;

end.
