{ File    : $URL: file:///D:/SVN/svnDelphi/trunk/DBIOS/src/utils/ModInfo.pas $
  Author  : $Author: Deniz Oezmen $
  Changed : $Date: 2020-03-13 10:46:05 +0100 (Fr, 13 Mrz 2020) $
  Revision: $Rev: 539 $
}
unit ModInfo;

interface

type
  TModuleInfo = array of String;

function GetModuleInfo: TModuleInfo;
function GetModuleFileVersionString: AnsiString;
function GetModuleNamePrefix: AnsiString;

implementation

uses
  SysUtils,
  Windows;

type
  TStringFileInfo = record
    CompanyName     : AnsiString;
    FileDescription : AnsiString;
    FileVersion     : AnsiString;
    InternalName    : AnsiString;
    LegalCopyright  : AnsiString;
    OriginalFileName: AnsiString;
    ProductName     : AnsiString;
    ProductVersion  : AnsiString;
  end;

var
  ModuleStringFileInfo: TStringFileInfo;

function GetStringFileInfo(const FileName: AnsiString): TStringFileInfo;
var
  Handle      : THandle;
  InfoSize    : Longword;
  FileInfo    : Pointer;
  Size        : Longword;
  Translation,
  Value       : Pointer;

  function GetValue(Block: AnsiString): AnsiString;
  begin
    if VerQueryValue(FileInfo, PChar(Format('\StringFileInfo\%.4x%.4x\' + Block,
      [LoWord(Longword(Translation^)), HiWord(Longword(Translation^))])), Value,
      Size) then
        Result := AnsiString(PChar(Value))
    else
      Result := ''
  end;

begin
  with Result do
  begin
    CompanyName      :='';
    FileDescription  :='';
    FileVersion      :='';
    InternalName     :='';
    LegalCopyright   :='';
    OriginalFileName :='';
    ProductName      :='';
    ProductVersion   :=''
  end;

  InfoSize := GetFileVersionInfoSize(PChar(FileName), Handle);
  if InfoSize > 0 then
    FileInfo := GetMemory(InfoSize)
  else
    FileInfo := nil;

  if Assigned(FileInfo) then
  begin
    if GetFileVersionInfo(PChar(FileName), Handle, InfoSize, FileInfo) then
    begin
      if VerQueryValue(FileInfo, '\VarFileInfo\Translation', Translation,
        Size) then
        with Result do
        begin
          CompanyName      := GetValue('CompanyName');
          FileDescription  := GetValue('FileDescription');
          FileVersion      := GetValue('FileVersion');
          InternalName     := GetValue('InternalName');
          LegalCopyright   := GetValue('LegalCopyright');
          OriginalFileName := GetValue('OriginalFileName');
          ProductName      := GetValue('ProductName');
          ProductVersion   := GetValue('ProductVersion')
        end
    end;

    FreeMemory(FileInfo)
  end
end;

function GetModuleStringFileInfo: TStringFileInfo;
var
  FileName: array[0..MAX_PATH] of Char;
begin
  GetModuleFileName(0, FileName, MAX_PATH);

  Result := GetStringFileInfo(FileName)
end;

function GetModuleInfo: TModuleInfo;
begin
  SetLength(Result, 3);

  with ModuleStringFileInfo do
  begin
    Result[0] := Format('%s     %s', [ProductName, FileVersion]);
    Result[1] := Format('%s', [LegalCopyright]);
    Result[2] := Format('%s', [CompanyName])
  end
end;

function GetModuleFileVersionString: AnsiString;
begin
  with ModuleStringFileInfo do
    Result := Trim(Format('%s %s', [InternalName, FileVersion]))
end;

function GetModuleNamePrefix: AnsiString;
var
  Name: AnsiString;
begin
  Name := ExtractFileName(GetModuleName(0));
  Result := Copy(Name, 1, Length(Name) - Length(ExtractFileExt(Name)))
end;

begin
  ModuleStringFileInfo := GetModuleStringFileInfo
end.
